from random import choice, random

from twistit.model import GameField, BaseBlock

def generate_block(sides):
    '''Generate a block which always has at least two different numbers'''
    block_sides = []
    while len(set(block_sides)) < 2:
        block_sides = [choice(sides) for x in range(4)]
    return BaseBlock(block_sides)


def shuffle(field):
    # Shuffle the field a thousand times max to make it a puzzle. By
    # not having a while this avoids creating an infinite loop.
    rotations = dict([((x, y), 0) 
                      for x in range(field.size_x) 
                      for y in range(field.size_y)])

    steps = {0: 0, 90: 1, 180: 2, 270: 1}

    for i in range(1000):
        if field.is_solved():
            for x in range(field.size_x):
                for y in range(field.size_y):
                    degrees = choice((0, 90, 180, 270))
                    rotations[(x, y)] += degrees
                    field.getblock(x, y).rotate(degrees)
        else:
            return sum([steps[rotation % 360] for rotation in rotations.values()])

    # At least raise an error when it cannot be shuffled
    raise ValueError('Field was not shuffled correctly after a 1000 tries:\n' 
                     % str(field))


def generate_field(size, sides, jokers=0, field_factory=GameField):
    field = field_factory(size)
    for x in range(size[0]):
        for y in range(size[1]):
            block = generate_block(sides)

            # if x is not the first block set the sides to match with
            # the previous
            if x: 
                block.left = field.getblock(x - 1, y).right
            # if y is not the first block set the sides to match with
            # the previous
            if y:
                block.top = field.getblock(x, y - 1).bottom

            field.setblock(block, x, y)

    for i in range(jokers):
        for j in range(100):
            side = int(random() * 4)
            x = int(random() * field.size_x)
            y = int(random() * field.size_y)
            
            block = field.getblock(x, y)
            if block.sides[side] != 'j':
                block.sides = block.sides[:side] + ('j', ) + block.sides[side + 1:]
                break

    return field
